/************************************************************************************************
 * Copyright ©, 2018-2020, MapGIS
 * @Description: 地图模块加载器
 * @Author: Chenzilong
 * @History: 1、初始版本 2018-11-19 Chenzilong
 * @Usage:  高阶组件的实用  => 通过封装一个高阶组件，先加载地图相对应的模块，加载完成，再来加载地图资源，这样在封装的组件里面就不会有异步过程，并且还使用了全局变量来进行组件缓存组件
 *
 * //加载模块
 * const modules = { Graphic:"esri/Graphic",GraphicsLayer:"esri/layers/GraphicsLayer" }; //需要加载的地图模块定义
 * 1、class修饰符方式
 * @withModule(modules) //使用修饰符修饰组件
 * class SomeComponent extends React.Component{}
 * 2、函数调用方式
 * const WrappedComponent = withModule(modules)(SomeComponent)
 * //使用组件
 * <WrappedComponent wrappedComponentRef={ref=>this.ref = ref}/>  //获取组件实例引用使用wrappedComponentRef
 * //获取模块
 * 组件内部通过 const {module:{Graphic,GraphicsLayer}} = this.props 获取被加载模块
 *
 * withModule会在加载完所有模块后显示组件，并将模块通过props.module注入到组件中
 ************************************************************************************************/

import React from 'react';
import { loadMapModules, getMapModules } from './loadMapModules';

const withModule = (modules) => {
  return function wrapper(Component) {
    class Proxy extends React.Component {
      state = {
        loaded: false,
      };

      unmounted = false;

      componentDidMount() {
        if (modules) {
          this.loadModules();
        } else {
          this.setState({ loaded: true });
        }
      }

      componentWillUnmount() {
        this.unmounted = true;
      }

      async loadModules() {
        await loadMapModules(modules);
        if (!this.unmounted) this.setState({ loaded: true });
      }

      render() {
        const { wrappedComponentRef, ...rest } = this.props;
        return this.state.loaded ? (
          <Component modules={getMapModules()} {...rest} ref={wrappedComponentRef} />
        ) : null;
      }
    }
    return Proxy;
  };
};

export default withModule;
