import React from 'react';
import amapConfig from './config/amap';
import { Spin } from 'antd';
import { layerOperation, loadUtilMapModules, geomUtils, mapUtils } from './core/utils';
import {
  loadMapLayerModules,
  AMapTileLayer,
  BaiduTileLayer,
  GoogleTileLayer,
  MapBoxLayer,
  PipenetTileLayer,
  TiandituTileLayer,
  UserTileLayer,
  WMTSTileLayer,
} from './extensions/layers/index';
import withModule from './core/withModule';
import loadScripts from './core/loadScripts';
const _ = require('lodash');

const modules = {
  Map: 'gis/Map',
  Basemap: 'gis/Basemap',
  MapView: 'gis/views/MapView',
  SceneView: 'gis/views/SceneView',
  TileLayer: 'gis/layers/TileLayer',
  MapImageLayer: 'gis/layers/MapImageLayer',
  ElevationLayer: 'gis/layers/ElevationLayer',
  Ground: 'gis/Ground',
  SceneLayer: 'gis/layers/SceneLayer',
  FeatureLayer: 'gis/layers/FeatureLayer',
  GroupLayer: 'gis/layers/GroupLayer',
  VectorTileLayer: 'gis/layers/VectorTileLayer',
  OpenStreetMapLayer: 'gis/layers/OpenStreetMapLayer',
  Extent: 'gis/geometry/Extent',
  Collection: 'gis/core/Collection',
  Camera: 'gis/Camera',
  Viewpoint: 'gis/Viewpoint',
  jsonUtils: 'gis/geometry/support/jsonUtils',
  watchUtils: 'gis/core/watchUtils',
  Graphic: 'gis/Graphic',
  PictureMarkerSymbol: 'gis/symbols/PictureMarkerSymbol',
  Popup: 'gis/widgets/Popup',
  GraphicsLayer: 'gis/layers/GraphicsLayer',
  matrix: 'dojox/gfx/matrix',
  Color: 'gis/Color',
  SimpleFillSymbol: 'gis/symbols/SimpleFillSymbol',
  SimpleLineSymbol: 'gis/symbols/SimpleLineSymbol',
  TextSymbol: 'gis/symbols/TextSymbol',
  Point: 'gis/geometry/Point',
  request: 'gis/request',
};

@withModule(modules)
export default class Map extends React.Component {
  static defaultProps = {
    id: '#jsmap',
    style: {
      width: '100%',
      height: '420px',
    },
    mapsettings: {},
    viewRef: React.createRef(),
  };

  state = {
    viewReady: true, //视图准备的条件 ： 1.has map 2. has container 3. extent
    loaded: true, //显示地图  create view
  };

  constructor(props) {
    super(props);
    this.initializeVariables();
  }

  initializeVariables = () => {
    this.view = null;
    this.map = null;
    this.basemap = null;
    this.tipManager = null;
    this.AreaLayerSum = null;
    this.hidetipHandler = null;
    this.showtipHandler = null;
    this.reorderGroups = [];
    this.basemaps = [];
    this.mapsettings = this.props.mapsettings;
    this.setInitialViewpoint = this.setInitialViewpoint.bind(this);
    this.layerAddedHandler = this.layerAddedHandler.bind(this);
    this.matrixLayerViewCreated = this.matrixLayerViewCreated.bind(this);
    this.matrixLayerMoveStart = this.matrixLayerMoveStart.bind(this);
    this.matrixLayerViewChange = this.matrixLayerViewChange.bind(this);
    this.matrixLayerMoveEnd = this.matrixLayerMoveEnd.bind(this);
  };

  loadAMapScript = () => {
    let amapUrl = `${amapConfig.url}v=${amapConfig.v}&key=${
      amapConfig.key
    }&plugin=${amapConfig.plugins.join(',')}`;
    return loadScripts([amapUrl]);
  };

  componentDidMount = async () => {
    await loadMapLayerModules();
    await loadUtilMapModules();
    this.renderView();
  };

  renderView = () => {
    this.addLayerProxy();
    this.addMap();
    this.addBasemaps();
    this.addLayerReorder();
    this.addLayers();
    this.addMatrixLayer();
    this.addTipManager();
    this.bindViewEvents();
  };

  addLayerProxy = () => {};

  changeState = (key, value) => {
    this.setState({ key, value });
  };

  addMap = () => {
    const { Map, Collection, MapView, Ground, SceneView } = this.props.modules;
    const {
      type,
      ground,
      resizeAlign,
      viewingMode,
      environment,
      clippingArea,
      extent,
      qualityProfile,
      constraints,
    } = this.mapsettings;
    const viewRef = this.props.viewRef;
    this.map = new Map();
    this.addGround({ ground, Ground, Collection });
    switch (type.toLowerCase()) {
      case '2d':
        this.createMapView({ MapView, viewRef, resizeAlign });
        break;
      case '3d':
        this.createMapSceneView({
          SceneView,
          viewRef,
          viewingMode,
          environment,
          clippingArea,
          extent,
          qualityProfile,
        });
        break;
      default:
        break;
    }
    if (constraints) this.view.constraints = constraints;
    this.viewpoint = this.mapsettings.viewpoint;
    this.view.then(this.setInitialViewpoint);
    this.changeState('loaded', true);
  };

  addGround = ({ ground, Ground, Collection }) => {
    if (ground) {
      if ('string' == typeof ground) {
        this.map.ground = ground;
      } else {
        let layers = ground.layers,
          groundLayers = new Collection();
        layers.map((layerConfig) => {
          this.createLayer(layerConfig, groundLayers, false, '');
        });
        this.map.ground = new Ground({
          layers: groundLayers,
        });
      }
    }
  };

  createMapView = ({ MapView, viewRef, resizeAlign }) => {
    this.view = new MapView({
      map: this.map,
      container: viewRef.current,
      manager: this,
      resizeAlign: resizeAlign || 'top-left',
    });
  };

  createMapSceneView = ({
    SceneView,
    viewRef,
    viewingMode,
    environment,
    clippingArea,
    extent,
    qualityProfile,
  }) => {
    this.view = new SceneView({
      map: this.map,
      manager: this,
      qualityProfile: qualityProfile || 'high',
      container: viewRef.current,
    });
    if (viewingMode) this.view.viewingMode = viewingMode;
    if (environment) this.view.environment = environment;
    if (clippingArea) this.view.clippingArea = clippingArea;
    if (extent) this.view.extent = extent;
  };

  setInitialViewpoint = () => {
    const { Viewpoint, Camera } = this.props.modules;
    if (this.viewpoint) {
      this.initialViewpoint = new Viewpoint();
      this.viewpoint.camera && (this.initialViewpoint.camera = new Camera(this.viewpoint.camera));
      this.viewpoint.targetGeometry &&
        (this.initialViewpoint.targetGeometry = geomUtils.toGeometry(
          this.viewpoint.targetGeometry,
          this.view,
        ));
      this.viewpoint.scale && (this.initialViewpoint.scale = this.viewpoint.scale);
      this.viewpoint.rotation && (this.initialViewpoint.rotation = this.viewpoint.rotation);
      let geom = this.initialViewpoint.targetGeometry;
      if (geom && !(geom.xmin == 0 && geom.ymin == 0 && geom.xmax == 0 && geom.ymax == 0)) {
        if (geom.type == 'extent') {
          let { zoom, scale } = mapUtils.getNearestScale(geom.clone(), this.view);
          if (!this.mapsettings.scale) {
            this.mapsettings.scale = scale;
            this.mapsettings.zoom = zoom;
          }
          if (scale != -1) {
            this.initialViewpoint.scale = scale;
          }
        }
      } else {
        if (this.view.extent) {
          this.initialViewpoint.targetGeometry = this.view.extent.clone();
          this.initialViewpoint.scale = this.view.scale;
        }
      }
      this.view.goTo(this.initialViewpoint);
    } else {
      this.initialViewpoint = this.view.viewpoint;
    }
    if (
      this.mapsettings.areasettings &&
      this.mapsettings.areasettings.areaName.toLowerCase().indexOf('offlinearea') > -1 &&
      !this.areaLayerInit
    ) {
      this.extent = {};
      this.setExtent();
      this.areaLayerInit = true;
      return;
    }
    this.mapsettings.useCoverMap = _.isUndefined(this.mapsettings.useCoverMap)
      ? true
      : !!this.mapsettings.useCoverMap;
    if (this.mapsettings.useCoverMap && !this.areaLayerInit) {
      this.loadAreaLayer();
    } else {
      this.showPipeNetLayer();
    }
  };

  addBasemaps = () => {
    let basemapconfigs = this.mapsettings.basemaps;
    if (basemapconfigs) {
      basemapconfigs.map((basemapconfig) => {
        this.basemaps.push(this.createBasemap(basemapconfig));
      });
    } else {
      this.basemaps.push(this.createDefaultBasemap());
    }
    if (this.basemaps.length) this.map.basemap = this.basemaps[0];
  };

  createDefaultBasemap = () => {};

  createBasemap = (basemap) => {
    const { Collection, Basemap } = this.props.modules;
    let bm;
    if (basemap) {
      if ('string' == typeof basemap) {
        bm = basemap;
      } else {
        let bls = new Collection(),
          id = basemap.id,
          title = basemap.title,
          thumbnailUrl = basemap.thumbnailUrl,
          baseLayers = basemap.baseLayers;
        baseLayers.map((layerconfig) => {
          this.createLayer(layerconfig, bls, false, 'baseMap');
        });
        bm = new Basemap({
          id,
          title,
          thumbnailUrl,
          baseLayers: bls,
        });
      }
    }
    return bm;
  };

  addLayerReorder = () => {
    this.map.layers.on('after-add', this.layerAddedHandler);
  };

  layerAddedHandler = (event) => {
    let lyr = event.item;
    let reorderGroups = this.reorderGroups;
    if ('graphics' == lyr.type) {
      if (layerOperation.isInGroup(lyr, reorderGroups)) {
        let layer = this.map.layers.find((layer) => {
          return (
            layerOperation.getLayerWeight(layer, reorderGroups) >
            layerOperation.getLayerWeight(lyr, reorderGroups)
          );
        });
        if (layer) {
          this.map.layers.reorder(lyr, this.map.layers.indexOf(layer) - 1);
        }
      }
    }
  };

  addLayers = () => {
    let layerconfigs = this.props.mapsettings.layers;
    if (layerconfigs) {
      layerconfigs.map((layerconfig) => {
        if (!layerconfig.id) return;
        this.createLayer(layerconfig, this.map, false, 'pipeLayer');
      });
    }
  };

  addMatrixLayer = () => {
    if (this.view.type != '2d') return;
    let { GraphicsLayer } = this.props.modules;
    this._matrixLayer = new GraphicsLayer();
    this.view.map.add(this._matrixLayer);
    this.view.whenLayerView(this._matrixLayer).then(this.matrixLayerViewCreated);
  };

  matrixLayerViewCreated = (lyrView) => {
    if (lyrView) {
      lyrView.on('moveStart', this.matrixLayerMoveStart);
      lyrView.on('viewChange', this.matrixLayerViewChange);
      lyrView.on('moveEnd', this.matrixLayerMoveEnd);
    }
  };

  matrixLayerMoveStart = (lyrView) => {
    let m = this.getLayerViewMatrix(lyrView);
    this.view.emit('moveStart', {
      layerView: lyrView,
      matrix: m,
      matrixString: this.toMatrixString(m),
    });
  };

  matrixLayerViewChange = (lyrView) => {
    let m = this.getLayerViewMatrix(lyrView);
    this.view.emit('viewChange', {
      layerView: lyrView,
      matrix: m,
      zooming: m.xx != 1,
      matrixString: this.toMatrixString(m),
    });
  };

  matrixLayerMoveEnd = (lyrView) => {
    let m = this.getLayerViewMatrix(lyrView);
    this.view.emit('moveEnd', {
      layerView: lyrView,
      matrix: m,
      matrixString: this.toMatrixString(m),
    });
  };

  getLayerViewMatrix = (lyrView) => {
    const { matrix } = this.props.modules;
    let m = matrix.clone(matrix.identify);
    if (
      lyrView.graphicsView &&
      lyrView.graphicsView._frontGroup &&
      lyrView.graphicsView._frontGroup.g &&
      lyrView.graphicsView._frontGroup.g.matrix
    ) {
      var targetMatrix = lyrView.graphicsView._frontGroup.g.matrix;
      if (_.isArray(targetMatrix)) {
        // 4.6版本 matrix为数组
        targetMatrix = {
          xx: targetMatrix[0],
          yx: targetMatrix[1],
          xy: targetMatrix[2],
          yy: targetMatrix[3],
          dx: targetMatrix[4],
          dy: targetMatrix[5],
        };
      }
      m = matrix.clone(targetMatrix);
    }
    return m;
  };

  toMatrixString = (m) => {
    return (
      'matrix(' +
      [
        m.xx.toFixed(8),
        m.yx.toFixed(8),
        m.xy.toFixed(8),
        m.yy.toFixed(8),
        m.dx.toFixed(8),
        m.dy.toFixed(8),
      ].join() +
      ')'
    );
  };

  addTipManager = () => {};

  bindViewEvents = () => {};

  loadAreaLayer = () => {
    let url = amapConfig.url;
    this.areaLayerInit = true;
    this.extent = {
      spatialReference: {
        wkid: 4526,
      },
      xmin: 0,
      ymin: 0,
      xmax: 0,
      ymax: 0,
    };
    let areasettings = this.mapsettings.areasettings;
    if (!areasettings) return;
    let area = areasettings.areaName.split(',');
    if (window.navigator.onLine == true && area.length == 1) {
      this.loadAMapScript()
        .then(() => {
          this.getAreaLayer();
        })
        .catch(() => new Error('加载高德地图资源失败'));
    } else {
      let pipenetLayer = this.getPipenetLayer();
      let url = pipenetLayer ? pipenetLayer.url : '';
      const { request } = this.props.modules;
      if (url && areasettings.areaName) {
        request(url + '/GetMapArea', {
          query: {
            areaName: area[0],
            outSR: mapUtils.getCurrentMapWkid(this.view),
          },
        })
          .then(
            function (r) {
              if (r.data && r.data.length > 0) {
                (this.polygon = []), (this.edge = []), (this.area = []), (this.bound = null);
                r.data.forEach(
                  function (point) {
                    this.polygon.push([point.x, point.y]);
                  }.bind(this),
                );
                this.edge.push(this.polygon);
                this.mapsettings.useCoverMap && this.area.push(this.polygon);
                this._setAreaLayer();
              } else {
                new Error('未查询到行政区数据');
              }
            }.bind(this),
          )
          .otherwise(
            function (error) {
              return new Error('未查询到行政区数据');
            }.bind(this),
          );
      }
    }
  };

  getAreaLayer = () => {
    let areaName = this.mapsettings.areasettings.areaName;
    if (!areaName) return new Error('未设置行政区域名称');
    let districtSearch = new AMap.DistrictSearch({
      // 关键字对应的行政区级别，country表示国家
      level: mapUtils.getAreaLevel(areaName),
      //  显示下级行政区级数，1表示返回下一级行政区
      subdistrict: 0,
      extensions: 'all',
    });
    let area = areaName.split(',');
    (this.edge = []), (this.area = []);
    districtSearch.search(
      area[0],
      function (status, result) {
        // 查询成功时，result即为对应的行政区信息
        this.extent.xmin = this.extent.ymin = this.extent.xmax = this.extent.ymax = 0;
        if (result.districtList && result.districtList.length > 0) {
          result.districtList.forEach(
            function (place) {
              if (place.boundaries.length > 0 && place.name.indexOf(area[0]) == 0) {
                place.boundaries.forEach(
                  function (data) {
                    this.polygon = [];
                    data.forEach(
                      function (point) {
                        let clonePoint = point;
                        // 若当前坐标wkid 为1021130 则需要转换为wgs84
                        if (mapUtils.getCurrentMapWkid(this.view) == '1021130') {
                          let transPoint = mapUtils.gcj2wgs({
                            Y: point.lng,
                            X: point.lat,
                          });
                          clonePoint = {
                            lng: transPoint.Y,
                            lat: transPoint.X,
                          };
                        }
                        var points = mapUtils.lonLat2WebMercator(clonePoint);
                        this.extent.xmin = this.extent.xmin == 0 ? points.x : this.extent.xmin;
                        this.extent.xmin =
                          points.x < this.extent.xmin ? points.x : this.extent.xmin;
                        this.extent.ymin = this.extent.ymin == 0 ? points.y : this.extent.ymin;
                        this.extent.ymin =
                          points.y < this.extent.ymin ? points.y : this.extent.ymin;
                        this.extent.xmax = this.extent.xmax == 0 ? points.x : this.extent.xmax;
                        this.extent.xmax =
                          points.x > this.extent.xmax ? points.x : this.extent.xmax;
                        this.extent.ymax = this.extent.ymax == 0 ? points.y : this.extent.ymax;
                        this.extent.ymax =
                          points.y > this.extent.ymax ? points.y : this.extent.ymax;
                        this.polygon.push([points.x, points.y]);
                      }.bind(this),
                    );
                    this.polygon = mapUtils.getProcessPoints(this.polygon, 100);
                    this.edge.push(this.polygon);
                    this.mapsettings.useCoverMap && this.area.push(this.polygon);
                  }.bind(this),
                );
              }
            }.bind(this),
          );
          if (this.edge.length > 0 && !this.coreArea) {
            this.bound = null;
            this._setAreaLayer();
          }
        } else {
          if (this.mapsettings.areasettings.extent) this.setExtent();
        }
      }.bind(this),
    );
  };

  _setAreaLayer = () => {
    const {
      GraphicsLayer,
      Graphic,
      SimpleFillSymbol,
      SimpleLineSymbol,
      Color,
    } = this.props.modules;
    let scale = 1,
      useCoverMap = this.mapsettings,
      areasettings = this.mapsettings.areasettings;
    let basemapId = this.view.map.basemap && this.view.map.basemap.id;
    this.AreaLayerSum = [];
    this.clearAreaLayer();
    if (basemapId && typeof this.mapsettings.areasettings.areaName !== '') {
      if (!this.areaLayer) {
        this.boundLayer = new GraphicsLayer({
          order: 'boundLayer',
          visible: false,
        });
        this.AreaLayerSum.push(this.boundLayer);
        this.view.map.add(this.boundLayer); //行政分区图层
        this.areaLayer = new GraphicsLayer({
          order: 'areaLayer',
          visible: false,
        });
        useCoverMap && this.AreaLayerSum.push(this.areaLayer);
        useCoverMap && this.view.map.add(this.areaLayer); //行政分区图层
      }
      if (!this.graphics) {
        this.graphics = [];
        var cover = new Graphic();
        var geometry = {
          hasM: 'false',
          hasZ: 'false',
          rings: this.area,
          spatialReference: {
            wkid: 4526,
          },
        };
        if (!this.bound) {
          this.bound = mapUtils.getBounds(basemapId);
          geometry.rings.push(this.bound);
        }
        cover.geometry = geomUtils.toGeometry(geometry, this.view);
        let color = Color.fromHex(
          areasettings && areasettings.backgroundColor != ''
            ? areasettings.backgroundColor
            : '#000',
        );
        cover.symbol = new SimpleFillSymbol({
          color: color,
          style: 'solid',
          join: 'round',
          outline: {
            width: 0,
            cap: 'round',
            join: 'round',
          },
        });
        let edge = new Graphic();
        edge.geometry = geomUtils.toGeometry(
          {
            hasM: 'false',
            hasZ: 'false',
            rings: this.edge,
            spatialReference: {
              wkid: 4526,
            },
          },
          this.view,
        );
        edge.symbol = new SimpleLineSymbol({
          color:
            areasettings && areasettings.boundColor != '' ? areasettings.boundColor : '#87fff4',
          width: areasettings && areasettings.boundWidth ? areasettings.boundWidth : 15 * scale,
          style: 'solid',
          join: 'round',
          outline: {
            width: 0,
            cap: 'round',
            join: 'round',
          },
        });
        this.graphics.push(cover);
        this.areaLayer.addMany(this.graphics);
        this.boundLayer.addMany([edge]);
        let opacity =
          areasettings && areasettings.backgroundOpacity !== ''
            ? areasettings.backgroundOpacity
            : 0.5;
        this.changeBackgroundOpacity(opacity);
        if (
          this.map.basemap &&
          this.map.basemap.baseLayers &&
          this.map.basemap.baseLayers.items &&
          this.map.basemap.baseLayers.items[0]
        ) {
          this.showCover();
          this.setExtent();
        } else {
          this.showPipeNetLayer();
        }
      }
    } else {
      new Error('底图未能正确加载，请检查运维配置');
    }
  };

  setExtent = () => {
    let geom = null,
      areasettings = this.mapsettings.areasettings;
    let extent = areasettings.extent;
    let targetGeometry = this.mapsettings.viewpoint.targetGeometry;
    if (extent) {
      [this.extent.min, this.extent.ymin, this.extent.xmax, this.extent.ymax] = _.split(
        extent,
        ',',
      );
      geom = this.initialViewpoint.targetGeometry = geomUtils.toGeometry(this.extent, this.view);
    } else {
      if (
        targetGeometry.xmin == targetGeometry.ymin &&
        targetGeometry.xmax == targetGeometry.ymax &&
        targetGeometry.ymin == targetGeometry.xmax
      ) {
        geom = this.initialViewpoint.targetGeometry = geomUtils.toGeometry(this.extent, this.view);
      }
    }
    if (geom) {
      if (geom && !(geom.xmin == 0 && geom.ymin == 0 && geom.xmax == 0 && geom.ymax == 0)) {
        if (geom.type == 'extent') {
          var scale = mapUtils.getNearestScale(geom.clone(), this.view);
          if (scale != -1) {
            this.initialViewpoint.scale = scale;
          }
        }
      } else {
        this.initialViewpoint.scale = this.view.scale;
      }
      if (
        !this.areaLayerInit &&
        areasettings &&
        areasettings.areaName.toLowerCase().indexOf('offlinearea') > -1
      ) {
        this.view.extent = this.initialViewpoint.targetGeometry.extent;
        this.view.scale = this.initialViewpoint.scale;
      }
      this.view.goTo(this.initialViewpoint);
      this.showPipeNetLayer();
    }
    this.showPipeNetLayer();
  };

  clearAreaLayer = () => {
    this.graphics = this.poi = null;
    this.areaLayer && this.areaLayer.removeAll() && this.view.map.remove(this.areaLayer); //行政区图层
    this.boundLayer && this.boundLayer.removeAll() && this.view.map.remove(this.boundLayer);
    this.areaLayer = null; //行政区图层
    this.boundLayer = null;
  };

  changeBackgroundOpacity = (opacity) => {
    if (this.areaLayer) this.areaLayer.opacity = parseFloat(opacity);
  };

  showCover = () => {
    this.AreaLayerSum &&
      this.AreaLayerSum.forEach(
        function (layer) {
          layer.visible = true;
        }.bind(this),
      );
  };

  createLayer = (config, collection, returnLayer, mapType) => {
    let layer,
      layers = [];
    let proxyUrl, configString, configv, configva, configi, configia;
    let layerType = config.layerType.toLowerCase();
    let _config = window._config || {};
    const {
      TileLayer,
      MapImageLayer,
      ElevationLayer,
      SceneLayer,
      FeatureLayer,
      GroupLayer,
      OpenStreetMapLayer,
    } = this.props.modules;
    config.url = this.getLayerUrl(config.url);
    switch (layerType) {
      case 'tilelayer':
        layer = new TileLayer(config);
        break;
      case 'pipenetlayer':
      case 'mapimagelayer':
        layer = new MapImageLayer(
          _.extend({}, config, {
            token: _config && _config.token,
            client: 'webgis',
            visible: false,
          }),
        );
        layer.then(
          function () {
            if (layer.hasAnnotation) {
              layer.annotationLayer = this.addAnnotationLayer(
                layer.title,
                layer.url,
                config.visible,
                _config,
              );
            }
          }.bind(this),
        );
        break;
      case 'elevationlayer':
        layer = new ElevationLayer(config);
        break;
      case 'scenelayer':
        layer = new SceneLayer(config);
        break;
      case 'featurelayer':
        layer = new FeatureLayer(config);
        break;
      case 'grouplayer':
        layers = config.layers.map(function (c) {
          return this.createLayer(c, [], true, '');
        }, this);
        layer = new GroupLayer(
          _.extend({}, config, {
            layers: layers,
          }),
        );
        break;
      case 'google-v':
      case 'google-i':
      case 'google-i-a':
      case 'google-ia':
        layer = new GoogleTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'google-user':
        proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new UserTileLayer()(
          _.extend(config, {
            url: proxyUrl,
          }),
        );
        break;
      case 'pipenet-tile':
        proxyUrl = config.proxyUrl ? config.proxyUrl : config.wmtsUrl ? config.wmtsUrl : null;
        layer = new PipenetTileLayer()(
          _.extend(config, {
            url: proxyUrl,
          }),
        );
        break;
      case 'baidu-v':
      case 'baidu-i':
      case 'baidu-ia':
        layer = new BaiduTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'amap-v':
      case 'amap-i':
      case 'amap-ia':
      case 'amap-v2':
      case 'amap-i2':
      case 'amap-ia2':
        layer = new AMapTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v':
      case 'tianditu-va':
      case 'tianditu-i':
      case 'tianditu-ia':
      case 'tianditu-t':
      case 'tianditu-ta':
      case 'tianditu-ll-v':
      case 'tianditu-ll-va':
      case 'tianditu-ll-i':
      case 'tianditu-ll-ia':
      case 'tianditu-ll-t':
      case 'tianditu-ll-ta':
        layer = new TiandituTileLayer()(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'tianditu-v-va':
        //深拷贝config
        configString = JSON.stringify(config);
        //天地图矢量
        configv = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-v',
        });
        this.createLayer(configv, collection, false, layerType);

        //天地图注记
        configva = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-va',
        });
        this.createLayer(configva, collection, false, layerType);
        break;
      case 'tianditu-i-ia':
        //深拷贝config
        configString = JSON.stringify(config);
        //天地图矢量
        configi = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-i',
        });
        this.createLayer(configi, collection, false, layerType);
        //天地图注记
        configia = _.extend(JSON.parse(configString), {
          layerType: 'tianditu-ia',
        });
        this.createLayer(configia, collection, false, layerType);
        break;
      case 'osm':
        layer = new OpenStreetMapLayer(
          _.extend(config, {
            url: 'http://fack.com',
          }),
        );
        break;
      case 'wmts-v':
        layer = new WMTSTileLayer()(_.extend(config, {}));
        break;
      case 'arcgis-streetpurplishblue':
      case 'arcgis-streetwarm':
      case 'arcgis-community':
      case 'arcgis-streetgray':
        layer = new TileLayer(
          _.extend(config, {
            url:
              'http://map.geoq.cn/ArcGIS/rest/services/ChinaOnline' +
              config.layerType.replace('arcgis-', '') +
              '/MapServer',
          }),
        );
        break;
      case 'arcgis-tilelayer':
        if (_config.ip && _config.ip.indexOf('222.92') > -1)
          config.url = config.url.replace(/\d+\.\d+\.\d+\.\d+/, window.location.hostname);
        layer = new TileLayer({
          url: config.url,
          title: config.title,
          id: _.uniqueId('arcgis-tilelayer_'),
        });
        break;
      default:
        if (layerType.indexOf('tianditu-') > -1) {
          layer = new TiandituTileLayer(
            _.extend(config, {
              url: 'http://fack.com',
            }),
          );
          break;
        }
        break;
    }
    if (returnLayer) return layer;
    if (_.isUndefined(layer)) return this._layerCreateErrorLog(config);
    switch (mapType) {
      case 'baseMap':
        layer.visible = true;
        break;
      case 'pipeLayer':
        _.extend(layer, {
          preload: true,
        });
        break;
      default:
        break;
    }
    collection.add(layer);
  };

  _layerCreateErrorLog = (layerconfig) => {
    var error =
      "mapView - can't createlayer (title:" +
      layerconfig.title +
      ',type:' +
      layerconfig.layerType +
      ',url:' +
      layerconfig.url +
      ')';
    console.log(error);
  };

  getLayerUrl = (url) => {
    if (!url) return '';
    return `${origin}/${url}`;
  };

  addAnnotationLayer = (title, url, visible, _config) => {
    let query = url.split('?')[1];
    const { MapImageLayer } = this.props.modules;
    url += query ? '&annotation=true' : 'annotation=true';
    let annotationLayer = new MapImageLayer({
      title: title + '（注记）',
      url: annotationLayerUrl,
      visible: visible,
      token: _config && _config.token,
      client: 'webgis',
    });
    _.extend(annotationLayer, {
      preload: true,
    });
    this.view.map.add(annotationLayer);
    return annotationLayer;
  };

  showPipeNetLayer = () => {
    var layers = this.map && this.map.layers;
    if (layers && layers.items) {
      layers.items.forEach(function (lyr) {
        if ('pipenetlayer' === lyr.layerType.toLowerCase()) {
          lyr.visible = true;
        }
      });
    }
  };

  render = () => {
    const { id, viewRef, style } = this.props;
    const { viewReady } = this.state;
    return (
      <div id={id} ref={viewRef} style={style}>
        {viewReady ? '' : <Spin tip="正在加载底图中..." />}
      </div>
    );
  };
}
